Package: ECDFniche
Type: Package
Title: Empirical Cumulative Distribution Function Niche Modeling Tools
Version: 0.5
Authors@R: c(
    person("Luíz Fernando", "Esser", , "luizesser@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-2982-7223")),
    person("Matheus", "Baumgartner", role = "aut",
           comment = c(ORCID = "0000-0001-7472-8588")),
    person("Dayani", "Bailly", role = "aut",
           comment = c(ORCID = "0000-0002-6954-9902")),
    person("Marcos", "R. Lima", role = "aut",
           comment = c(ORCID = "0000-0002-5901-0911")),
    person("Reginaldo", "Ré", role = "aut",
           comment = c(ORCID = "0000-0001-6452-3466")))
Description: Simulate ecological niche models using Mahalanobis distance, transform distances to 
    suitability with 1 - empirical cumulative distribution function and 1 - chi-squared, and 
    generate comparison figures.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: checkCLI, dplyr, ggpp, purrr, tidyr, ggplot2, lemon, MASS,
        stats
Suggests: knitr, rmarkdown, roxyglobals, tictoc
VignetteBuilder: knitr
Config/roxyglobals/filename: generated-globals.R
Config/roxyglobals/unique: TRUE
URL: https://luizesser.github.io/ECDFniche/
NeedsCompilation: no
Packaged: 2026-04-27 18:21:33 UTC; luizesser
Author: Luíz Fernando Esser [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2982-7223>),
  Matheus Baumgartner [aut] (ORCID:
    <https://orcid.org/0000-0001-7472-8588>),
  Dayani Bailly [aut] (ORCID: <https://orcid.org/0000-0002-6954-9902>),
  Marcos R. Lima [aut] (ORCID: <https://orcid.org/0000-0002-5901-0911>),
  Reginaldo Ré [aut] (ORCID: <https://orcid.org/0000-0001-6452-3466>)
Maintainer: Luíz Fernando Esser <luizesser@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 18:40:09 UTC
Built: R 4.6.0; ; 2026-04-27 21:39:22 UTC; unix

Package: ECDFniche
Type: Package
Title: Empirical Cumulative Distribution Function Niche Modeling Tools
Version: 0.5
Authors@R: c(
    person("Luíz Fernando", "Esser", , "luizesser@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-2982-7223")),
    person("Matheus", "Baumgartner", role = "aut",
           comment = c(ORCID = "0000-0001-7472-8588")),
    person("Dayani", "Bailly", role = "aut",
           comment = c(ORCID = "0000-0002-6954-9902")),
    person("Marcos", "R. Lima", role = "aut",
           comment = c(ORCID = "0000-0002-5901-0911")),
    person("Reginaldo", "Ré", role = "aut",
           comment = c(ORCID = "0000-0001-6452-3466")))
Description: Simulate ecological niche models using Mahalanobis distance, transform distances to 
    suitability with 1 - empirical cumulative distribution function and 1 - chi-squared, and 
    generate comparison figures.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: checkCLI, dplyr, ggpp, purrr, tidyr, ggplot2, lemon, MASS,
        stats
Suggests: knitr, rmarkdown, roxyglobals, tictoc
VignetteBuilder: knitr
Config/roxyglobals/filename: generated-globals.R
Config/roxyglobals/unique: TRUE
URL: https://luizesser.github.io/ECDFniche/
NeedsCompilation: no
Packaged: 2026-04-27 18:21:33 UTC; luizesser
Author: Luíz Fernando Esser [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2982-7223>),
  Matheus Baumgartner [aut] (ORCID:
    <https://orcid.org/0000-0001-7472-8588>),
  Dayani Bailly [aut] (ORCID: <https://orcid.org/0000-0002-6954-9902>),
  Marcos R. Lima [aut] (ORCID: <https://orcid.org/0000-0002-5901-0911>),
  Reginaldo Ré [aut] (ORCID: <https://orcid.org/0000-0001-6452-3466>)
Maintainer: Luíz Fernando Esser <luizesser@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 18:40:09 UTC
Built: R 4.6.0; ; 2026-05-01 21:56:26 UTC; unix

# Generated by roxygen2: do not edit by hand

export(create_distance_suitability_plot)
export(ecdf_compare_niche)
export(ecdf_nonnormal_niche)
export(ecdf_theoretical_niche)
export(mahal.dist)
export(run_ecdf_mahal_analysis)
import(checkCLI)
importFrom(MASS,mvrnorm)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,sample_frac)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_pointrange)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,label_both)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_colour_viridis_c)
importFrom(ggplot2,stat_function)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggpp,geom_plot_npc)
importFrom(lemon,grid_arrange_shared_legend)
importFrom(purrr,map)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,ecdf)
importFrom(stats,mahalanobis)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,qweibull)
importFrom(stats,rWishart)
importFrom(stats,sd)
importFrom(tidyr,nest)

# ECDFniche

<!-- badges: start -->
<!-- badges: end -->

Luíz Fernando Esser

ECDFniche is an R package for converting environmental distances into probability-like values using empirical cumulative distribution functions (ECDF), aimed at ecological niche and species distribution modeling workflows.

## Package overview

ECDFniche implements an ECDF-based mapping from distances in environmental space to values on a 0–1 scale that behave like cumulative probabilities.
The package is designed primarily for presence-only niche modeling, where distance-based algorithms (e.g., Mahalanobis, Euclidean or other metrics) need a consistent way to convert distances into suitability or probability-like surfaces.
The package is motivated by the classic use of Mahalanobis distance and its mapping to a chi-squared distribution to obtain probabilities, while removing the need for strict parametric assumptions.
By relying on the empirical cumulative distribution of observed distances, ECDFniche can be used with multiple distance metrics and remains flexible when the theoretical distribution of the distance is unknown or violated.

## Installation

You can install the development version of ECDFniche from [GitHub](https://github.com/luizesser/ECDFniche) with:

``` r
install.packages("devtools")
devtools::install_github("luizesser/caretSDM")
```

The package is also available on CRAN. Users are able to install it using the following code:

``` r
install.packages("caretSDM")
```

# Citation

If you use ECDFniche in a publication, please cite the associated methodological paper on ECDF-based distance-to-suitability mapping in ecological niche modeling once available.

#' Mahalanobis Distance Classifier for Ecological Niche Modeling
#'
#' A custom \code{caret} model specification implementing a Mahalanobis
#' distance-based classifier for ecological niche modeling (ENM) and
#' species distribution modeling (SDM). This implementation supports both
#' parametric (chi-squared) and nonparametric (empirical cumulative
#' distribution function; ECDF) transformations of Mahalanobis distances
#' into suitability scores.
#'
#' The model is trained using presence-only data to estimate the centroid
#' and covariance structure of environmental conditions associated with
#' species occurrences. Suitability is then derived as the inverse tail
#' probability of the Mahalanobis distance between new observations and
#' the estimated niche centroid.
#'
#' Two approaches are available to transform Mahalanobis distances into
#' probabilities:
#' \itemize{
#'   \item \code{"chisq"}: assumes distances follow a chi-squared
#'   distribution with degrees of freedom equal to the number of predictors.
#'   \item \code{"ecdf"}: uses the empirical cumulative distribution
#'   function of training distances, providing a nonparametric estimate
#'   of suitability.
#' }
#'
#' The ECDF-based approach is particularly useful when the assumption of
#' multivariate normality is violated, which is common in ecological data.
#'
#' This model can be used within the \code{caret::train()} framework,
#' enabling resampling, tuning, and ensemble modeling workflows for
#' ecological niche modeling.
#'
#' @section Model Parameters:
#' \describe{
#'   \item{abs}{Logical. If \code{TRUE}, predictions are binarized using a
#'   fixed threshold (default: 0.05). If \code{FALSE}, the class with the
#'   highest predicted probability is returned.}
#'
#'   \item{method}{Character. Method used to convert Mahalanobis distances
#'   into suitability values. Options are \code{"chisq"} or \code{"ecdf"}.}
#' }
#'
#' @section Details:
#' The Mahalanobis distance defines an ellipsoidal niche in environmental
#' space. Under the chi-squared formulation, suitability decreases as the
#' distance from the niche centroid increases. The ECDF formulation
#' relaxes distributional assumptions by estimating suitability directly
#' from the empirical distribution of distances observed in presence data.
#'
#' Predictions return class probabilities for \code{"presence"} and
#' \code{"pseudoabsence"}, allowing flexible thresholding and ensemble
#' integration.
#'
#' @section Usage in caret:
#' This object can be supplied to \code{caret::train()} as a custom model:
#'
#' \preformatted{
#' library(caret)
#'
#' model <- train(
#'   x = predictors,
#'   y = response,
#'   method = mahal.dist,
#'   trControl = trainControl(classProbs = TRUE)
#' )
#' }
#'
#' You can also run only ECDF by adjusting the tuning grid:
#' \preformatted{
#' library(caret)
#'
#' grid <- expand.grid(
#'   abs = c(TRUE, FALSE),
#'   method = "ecdf"
#' )
#'
#' model <- train(
#'   x = predictors,
#'   y = response,
#'   method = mahal.dist,
#'   tuneGrid = grid,
#'   trControl = trainControl(classProbs = TRUE)
#' )
#' }
#'
#' @seealso \code{\link[stats]{mahalanobis}}, \code{\link[stats]{ecdf}},
#'   \code{\link[caret]{train}}
#'
#' @keywords models niche-modeling species-distribution-modeling mahalanobis
#'
#' @importFrom stats cov mahalanobis pchisq ecdf
#' @export
mahal.dist <- list(

  label = "Mahalanobis Distance Classifier",
  library = NULL,
  type = "Classification",

  parameters = data.frame(
    parameter = c("abs", "method"),
    class = c("logical", "character"),
    label = c("Absolute Binarization", "Suitability method")
  ),

  grid = function(x, y, len = NULL, search = "grid") {
    expand.grid(
      abs = c(TRUE, FALSE),
      method = c("chisq", "ecdf")
    )
  },

  fit = function(x, y, wts, param, lev, last, classProbs, ...) {

    # Use only presence data
    presence_data <- x[y == "presence", , drop = FALSE]

    if (nrow(presence_data) < 2) {
      stop("Not enough 'presence' data points to calculate covariance.")
    }

    # Core Mahalanobis parameters
    center_vec <- colMeans(presence_data, na.rm = TRUE)
    cov_mat <- stats::cov(presence_data)
    inv_cov_matrix <- solve(cov_mat)

    # Training distances (for ECDF)
    d2_train <- stats::mahalanobis(
      x = presence_data,
      center = center_vec,
      cov = inv_cov_matrix,
      inverted = TRUE
    )

    # Store model
    result <- list(
      center = center_vec,
      inv_cov = inv_cov_matrix,
      df = ncol(x),
      abs = param$abs,
      method = param$method,
      ecdf_fun = stats::ecdf(d2_train),
      levels = lev
    )

    return(result)
  },

  predict = function(modelFit, newdata, preProc = NULL, submodels = NULL) {

    probs <- mahal.dist$prob(modelFit, newdata)

    if (modelFit$abs) {
      pred <- ifelse(
        probs[, modelFit$levels[1]] >= 0.05,
        modelFit$levels[1],
        modelFit$levels[2]
      )
    } else {
      pred <- colnames(probs)[apply(probs, 1, which.max)]
    }

    factor(pred, levels = modelFit$levels)
  },

  prob = function(modelFit, newdata, preProc = NULL, submodels = NULL) {

    d2 <- stats::mahalanobis(
      x = newdata,
      center = modelFit$center,
      cov = modelFit$inv_cov,
      inverted = TRUE
    )

    # Switch between methods
    if (modelFit$method == "chisq") {

      p_presence <- 1 - stats::pchisq(d2, df = modelFit$df)

    } else if (modelFit$method == "ecdf") {

      p_presence <- 1 - modelFit$ecdf_fun(d2)

    } else {
      stop("Unknown method: ", modelFit$method)
    }

    prob_df <- data.frame(
      presence = p_presence,
      pseudoabsence = 1 - p_presence
    )

    colnames(prob_df) <- modelFit$levels
    return(prob_df)
  },

  predictors = function(x, ...) {
    names(x$center)
  },

  varImp = function(object, ...) {
    # simple proxy: variance contribution
    data.frame(Overall = diag(solve(object$inv_cov)))
  },

  levels = function(x) x$levels,

  tags = c("mahalanobis", "Presence-Only", "ECDF", "Nonparametric")
)

#' Simulations and analyses of Mahalanobis distance-based habitat suitability
#'
#' Performs replicated simulations of multivariate normal data to evaluate
#' the agreement between suitability derived from chi-squared distribution
#' and empirical cumulative distribution function (ECDF).
#'
#' @param p_vals Integer vector; number of predictor variables (dimensions).
#' @param n_vals Integer vector; number of records (sample sizes).
#' @param n_reps Integer; number of replicates per combination.
#' @param seed Optional integer for reproducibility.
#'
#' @description
#' The objective is to compare the performance of habitat suitability calculated
#' based on chi-squared cumulative distribution function and Empirical
#' Cumulative Distribution Function (ECDF)
#'
#' @return A list with:
#' \itemize{
#'   \item cor_plot: ggplot of correlation vs sample size.
#'   \item suit_plot: ggplot of suitability vs Mahalanobis distance.
#'   \item cond_plot: ggplot of correlation vs condition number.
#'   \item cor_df: raw correlation data.
#'   \item obs_df: observation-level data.
#'   \item cov_df: covariance diagnostics.
#' }
#'
#' @author Matheus T. Baumgartner
#'
#' @examples
#' # Create ECDF-niche based on personalized options:
#' n <- ecdf_compare_niche(p_vals = 1:3,
#'                         n_vals = seq(50L, 500L, 50L),
#'                         n_reps = 10L,
#'                         seed = 1991)
#'
#' @importFrom stats rWishart mahalanobis cov pchisq ecdf cor cov2cor sd
#' @importFrom MASS mvrnorm
#' @importFrom dplyr group_by summarise mutate ungroup sample_frac
#' @importFrom ggplot2 ggplot geom_point geom_pointrange stat_function
#'   facet_wrap labs theme_bw theme element_blank element_text scale_color_manual
#'   label_both
#' @import checkCLI
#'
#' @global cor
#'
#' @export
ecdf_compare_niche <- function(
    p_vals = 1:5,
    n_vals = seq(20L, 500L, 20L),
    n_reps = 30L,
    seed = NULL) {

  # Assertions
  assert_vector_cli(p_vals, min.len = 1, unique = TRUE)
  assert_numeric_cli(p_vals, lower = 1)
  assert_numeric_cli(n_vals, lower = 1)
  assert_vector_cli(n_vals, min.len = 1, unique = TRUE)
  assert_numeric_cli(n_reps, lower = 1)
  assert_numeric_cli(seed, lower = 0, null.ok = TRUE)

  if (!is.null(seed)) {
    set.seed(seed)
  }

  cor_df  <- list()
  obs_df  <- list()
  cov_df  <- list()

  idx_cor <- 1
  idx_obs <- 1
  idx_cov <- 1

  for (p in p_vals) {
    for (n in n_vals) {

      ## Generate a fixed variance-covariance matrix for this (p,n) combination
      ## by simulating from a Wishart distribution
      # - The seed depends on p an n so that all replicates share the same
      #   Sigma matrix
      # - df = p + 2 ensures positive definiteness for var-cov matrices
      # - scale = diag(p) gives a baseline for variances
      set.seed(p * 10 + n)
      Sigma <- stats::rWishart(1, df = p + 2, Sigma = diag(p))[,,1]

      ## Simulate for each replicate
      for (rep in seq_len(n_reps)) {

        # Set the seed for this replicate
        set.seed(p * 10 + n * 10 + rep)

        # Generate data from MVN(vec(0), Sigma)
        smp <- MASS::mvrnorm(n = n, mu = rep(0, p), Sigma = Sigma)

        # Compute sample mean and covariance matrix
        mu_hat <- colMeans(smp)
        sigma_hat <- stats::cov(smp)

        # Compute Mahalanobis distance (squared) for each observation
        # based on the
        D2 <- stats::mahalanobis(smp, center = mu_hat, cov = sigma_hat)

        # Compute theoretical suitability (chi-squared)
        S_chisq <- 1 - stats::pchisq(D2, df = p)

        # Compute empirical suitability based on ECDF
        S_ecdf  <- 1 - stats::ecdf(D2)(D2)

        # Compute Pearson correlation coefficient between the two suitabilities
        # and store correlation result in main data frame
        cor_df[[idx_cor]] <- data.frame(
          p = p,
          n = n,
          rep = rep,
          cor = stats::cor(S_chisq, S_ecdf)
        )

        # Store correlation result in main data frame
        # Store individual observations
        obs_df[[idx_obs]] <- data.frame(
          p = p,
          n = n,
          rep = rep,
          D2 = D2,
          S_chisq = S_chisq,
          S_ecdf = S_ecdf
        )

        ## Compute variance-covariance matrix properties
        eig <- eigen(sigma_hat, symmetric = TRUE, only.values = TRUE)$values
        # Condition Number
        cond_num <- max(eig) / min(eig)  # measure of multicollinearity

        # Average absolute off-diagonal values (covariances/correlations)
        avg_abs_cor <- if (p > 1) {
          mean(abs(stats::cov2cor(sigma_hat)[lower.tri(sigma_hat)]))
        } else NA

        # Store properties
        cov_df[[idx_cov]] <- data.frame(
          p = p,
          n = n,
          cond_num = cond_num,
          avg_abs_cor = avg_abs_cor
        )

        idx_cor <- idx_cor + 1
        idx_obs <- idx_obs + 1
        idx_cov <- idx_cov + 1
      }
    }
  }

  # Bind results
  cor_df <- do.call(rbind, cor_df)
  obs_df <- do.call(rbind, obs_df)
  cov_df <- do.call(rbind, cov_df)

  # ------------------------------------------------------------------------------
  # Graph 1: Correlation between theoretical and empirical suitabilities
  # ------------------------------------------------------------------------------

  # Summarize correlations based on (n,p) combinations: mean and sd across
  # all replicates
  cor_summary <- cor_df |>
    dplyr::group_by(p, n) |>
    dplyr::summarise(
      mean_cor = mean(cor),
      sd_cor = stats::sd(cor),
      .groups = "drop"
    )

  # Plot: correlation vs. sample size, with error bars, by p
  cor_plot <- ggplot2::ggplot() +
    ggplot2::ylim(c(0.94, 1)) +
    ggplot2::geom_hline(yintercept = 1,
                        col = "black",
                        linetype = "dashed") +
    ggplot2::geom_point(data = cor_df,
                        ggplot2::aes(x = .data$n,
                                     y = .data$cor),
                        shape = 1,
                        col = "grey") +
    ggplot2::geom_pointrange(data = cor_summary,
                             ggplot2::aes(x = .data$n,
                                          y = .data$mean_cor,
                             ymin = .data$mean_cor - .data$sd_cor,
                             ymax = .data$mean_cor + .data$sd_cor),
                    size = 0.5,
                    linewidth = 0.5,
                    colour = "red") +
    ggplot2::facet_wrap(. ~ .data$p, nrow = 1, labeller = ggplot2::label_both) +
    ggplot2::labs(x = expression("Number of records (n)"),
                  y = "Pearson's correlation") +
    ggplot2::theme_bw(base_size = 12) +
    ggplot2::theme(panel.grid.minor = ggplot2::element_blank())

  # ------------------------------------------------------------------------------
  # Graph 2: Suitability vs. Mahalanobis distance
  # ------------------------------------------------------------------------------

  # Summarize observations across all replicates and sample sizes, then compute
  # the empirical suitability function on the pooled D2 values
  plot_df <- obs_df |>
    dplyr::group_by(p) |>
    dplyr::mutate(S_ecdf_pooled = 1 - stats::ecdf(D2)(D2)) |> # global ECDF per p
    dplyr::ungroup()

  # For each observation, we have both the S_chisq and the pooled S_ecdf
  # (use only a proportion of points)
  suit_plot <- ggplot2::ggplot(dplyr::sample_frac(plot_df, 0.1), ## sample 10% of points for plotting
                               ggplot2::aes(x = .data$D2,
                                            y = .data$S_ecdf_pooled)) +
    ggplot2::geom_point(alpha = 0.1,
                        ggplot2::aes(color = "ECDF"), size = 1.2) +
    ggplot2::stat_function(fun = function(x) 1 - stats::pchisq(x, df = 1),
                  aes(color = "Chi-squared")) +
    ggplot2::stat_function(fun = function(x) 1 - stats::pchisq(x, df = 2),
                  colour = "grey") +
    ggplot2::stat_function(fun = function(x) 1 - stats::pchisq(x, df = 3),
                  colour = "grey") +
    ggplot2::stat_function(fun = function(x) 1 - stats::pchisq(x, df = 4),
                  colour = "grey") +
    ggplot2::stat_function(fun = function(x) 1 - stats::pchisq(x, df = 5),
                  colour = "grey") +
    ggplot2::facet_wrap(. ~ .data$p, nrow = 1, labeller = ggplot2::label_both) +
    ggplot2::labs(x = expression("Squared Mahalanobis distance" ~ (D^2)),
                  y = "Habitat suitability") +
    ggplot2::scale_color_manual(name = NULL,
                       values = c("ECDF" = "red",
                                  "Chi-squared" = "grey")) +
    ggplot2::theme_bw(base_size = 12) +
    ggplot2::theme(legend.position = "bottom",
          panel.grid.minor = ggplot2::element_blank(),
          axis.text = ggplot2::element_text(color = "black"))

  # ------------------------------------------------------------------------------
  # Graph 3: Mean correlation vs. condition number
  # ------------------------------------------------------------------------------

  # Merge the mean correlation with summary
  analysis_df <- merge(cor_summary, cov_df, by = c("p", "n"))

  # This measures the effect of collinearity on ECDF
  cond_plot <- ggplot2::ggplot(analysis_df,
                               ggplot2::aes(x = .data$cond_num,
                                            y = .data$mean_cor,
                                            colour = .data$n)) +
    ggplot2::ylim(c(0.94, 1)) +
    ggplot2::geom_point(size = 3) +
    ggplot2::facet_wrap(. ~ .data$p, nrow = 1, labeller = ggplot2::label_both) +
    ggplot2::scale_x_log10() +
    ggplot2::labs(x = "Condition number (log)",
                  y = "Pearson's correlation",
                  color = "Num. records (n)") +
    ggplot2::theme_bw(base_size = 12) +
    ggplot2::theme(legend.position = "bottom")

  # ------------------------------------------------------------------------------
  # Graph 4: Mean correlation vs. average absolute correlation (off-diagonal)
  # ------------------------------------------------------------------------------

  # Only for p > 1
  p_cor_avg <- ggplot2::ggplot(subset(analysis_df, analysis_df$p > 1),
                               ggplot2::aes(x = .data$avg_abs_cor,
                                            y = .data$mean_cor,
                                            color = .data$n)) +
    ggplot2::ylim(c(0.94, 1)) +
    ggplot2::geom_point(size = 3) +
    ggplot2::facet_wrap(. ~ .data$p, nrow = 1, labeller = ggplot2::label_both) +
    ggplot2::labs(x = "Average absolute off-diagonal correlation",
                  y = "Pearson's correlation",
                  color = "Num. records (n)") +
    ggplot2::theme_bw(base_size = 12) +
    ggplot2::theme(legend.position = "bottom")


  # Return all results in a list
  l <- list(
    cor_plot = cor_plot,
    suit_plot = suit_plot,
    cond_plot = cond_plot,
    cor_avg_plot = p_cor_avg,
    cor_df = cor_df,
    obs_df = obs_df,
    cov_df = cov_df
  )

  return(l)
}

#' Niche analysis using ECDF and chi-squared
#'
#' Simulate niche suitability from Mahalanobis distance using both
#' chi-squared and empirical CDF transformations, for a given number
#' of predictor variables.
#'
#' @param n Integer; number of predictor variables (dimensions).
#' @param n_population Integer; size of simulated environmental population.
#' @param sample_sizes Integer vector of sample sizes to evaluate.
#' @param seed Optional integer seed for reproducibility.
#'
#' @return A list with:
#' \itemize{
#'   \item corplot: ggplot object with correlation vs sample size.
#'   \item sample_data: matrix of simulated sample points.
#'   \item sample_niche: numeric vector of “true” niche suitability.
#'   \item chisq_suits: numeric vector, 1 - pchisq(Mahalanobis).
#'   \item ecdf_suits: numeric vector, 1 - ECDF(Mahalanobis).
#'   \item mahal_dists: numeric vector of Mahalanobis distances.
#' }
#'
#' @author Luíz Fernando Esser
#'
#' @examples
#' # Create ECDF-niche based on personalized options:
#' n <- ecdf_theoretical_niche(n = 3,
#'                             n_population = 20000,
#'                             sample_sizes = seq(50, 1000, 50),
#'                             seed = 123)
#'
#' @importFrom MASS mvrnorm
#' @importFrom ggplot2 ggplot aes geom_line theme_bw labs scale_colour_manual theme element_blank
#'             ylim
#' @importFrom stats pchisq ecdf cor mahalanobis cov
#' @import checkCLI
#'
#' @global
#'
#' @export
ecdf_theoretical_niche <- function(
    n,
    n_population = 10000L,
    sample_sizes = seq(20L, 500L, 20L),
    seed = NULL) {

  # Assertions
  assert_numeric_cli(n, len = 1, lower = 1)
  assert_numeric_cli(n_population, len = 1, lower = 1)
  assert_vector_cli(sample_sizes, min.len = 1, unique = TRUE)
  assert_numeric_cli(sample_sizes, lower = 1L, null.ok = TRUE)
  assert_numeric_cli(seed, lower = 0, null.ok = TRUE)

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # population
  population_grid <- lapply(seq_len(n), function(x) stats::rnorm(n_population))
  population_grid <- as.data.frame(do.call(cbind, population_grid))
  population_means <- colMeans(population_grid)
  population_covariance <- stats::cov(population_grid)

  ecdf_correlations  <- numeric(0L)
  chisq_correlations <- numeric(0L)

  # store last sample for returning
  last_sample_data         <- NULL
  last_niche_suitability   <- NULL
  last_chisq_suitabilities <- NULL
  last_ecdf_suitabilities  <- NULL
  last_mahalanobis         <- NULL

  for (current_sample_size in sample_sizes) {
    sample_data <- MASS::mvrnorm(
      n   = current_sample_size,
      mu  = population_means,
      Sigma = population_covariance
    )

    mahalanobis_distances <- stats::mahalanobis(
      x      = sample_data,
      center = population_means,
      cov    = population_covariance
    )

    niche_suitability <- 1 - stats::pchisq(mahalanobis_distances, df = n)

    chisq_suitabilities <- 1 - stats::pchisq(mahalanobis_distances, df = n)

    ecdf_function <- stats::ecdf(mahalanobis_distances)
    ecdf_suitabilities <- 1 - ecdf_function(mahalanobis_distances)

    correlation_matrix <- stats::cor(cbind(
      niche_suitability,
      chisq_suitabilities,
      ecdf_suitabilities
    ))

    ecdf_correlations  <- c(ecdf_correlations,  correlation_matrix[1, 3])
    chisq_correlations <- c(chisq_correlations, correlation_matrix[1, 2])

    last_sample_data         <- sample_data
    last_niche_suitability   <- niche_suitability
    last_chisq_suitabilities <- chisq_suitabilities
    last_ecdf_suitabilities  <- ecdf_suitabilities
    last_mahalanobis         <- mahalanobis_distances
  }

  plot_data <- data.frame(
    Sample_Size = c(sample_sizes, sample_sizes),
    Correlation = c(chisq_correlations, ecdf_correlations),
    Method      = c(
      rep("1 - Chi-squared", length(sample_sizes)),
      rep("1 - ECDF",       length(sample_sizes))
    )
  )

  correlation_plot <- ggplot2::ggplot(data = plot_data) +
    ggplot2::theme_bw() +
    ggplot2::geom_line(
      ggplot2::aes(x = .data$Sample_Size, y = .data$Correlation, colour = .data$Method),
      linewidth = 1.0,
      stat = "identity"
    ) +
    ggplot2::ylim(0, 1) +
    ggplot2::theme(
      legend.position = "bottom",
      legend.direction = "horizontal",
      legend.title = ggplot2::element_blank()
    ) +
    ggplot2::labs(
      x = "Number of Records",
      y = "Correlation",
      title = paste(n, "Predictor Variables")
    ) +
    ggplot2::scale_colour_manual(values = c("cyan4", "orange"))

  list(
    corplot     = correlation_plot,
    sample_data = last_sample_data,
    sample_niche = last_niche_suitability,
    chisq_suits  = last_chisq_suitabilities,
    ecdf_suits   = last_ecdf_suitabilities,
    mahal_dists  = last_mahalanobis
  )
}

#' Compare ECDF and Chi-square suitability under non-normal data
#'
#' Simulates bivariate environmental data using Gaussian copulas with
#' non-normal marginals (Normal for temperature and Weibull for precipitation),
#' and evaluates agreement between chi-squared and ECDF suitability.
#'
#' @param rho_vals Numeric vector; correlations between variables.
#' @param n_vals Integer vector; sample sizes.
#' @param n_reps Integer; number of replicates.
#' @param N_ref Integer; size of reference population for "true" parameters.
#' @param temp_function Character; function used to model temperature values. One of: "qnorm" or
#' "qweibull".
#' @param prec_function Character; function used to model precipitation values. One of: "qnorm" or
#' "qweibull".
#' @param temp_parameters  List; list organizing parameters to pass to \code{temp_function}.
#' @param prec_parameters List; list organizing parameters to pass to \code{temp_function}.
#' @param seed Optional integer for reproducibility.
#'
#' @description
#' Script to run a simulation study to compare Chi-square vs. ECDF approaches
#' to quantify habitat suitability based on bivariate non-normal data.
#' Bivariate data was simulated based on environmental variables (temperature
#' and precipitation) using Gaussian copulas. Temperature followed a normal
#' distribution while precipitation followed a Weibull distribution.
#' The choices of the distributions were based on Haddad (2021) - Theoretical
#' and Applied Climatology (for temperature) and on the estimation of rainfall
#' in milimeters by Wilks (1989) - Journal of Applied Meteorology.
#' Because the relationship between temperature and precipitation is complex
#' across space (Rodrigo, 2022 - Theoretical and Applied Climatology), we
#' defined five correlation values between the two variables.
#'
#' \code{temp_parameters} and \code{prec_parameters} must comply to \code{stats::qnorm} or
#' \code{stats::qweibull}, depending on the function chosen on \code{temp_function} and
#' \code{prec_function}. For \code{"qnorm"}, user can specify \code{mean} and \code{sd}, while
#' for \code{"qweibull"}
#'
#' @return A list with:
#' \itemize{
#'   \item suit_plot: ggplot of suitability vs Mahalanobis distance
#'   \item cor_df: correlation results
#'   \item obs_df: observation-level data
#' }
#'
#' @author Matheus T. Baumgartner
#'
#' @examples
#' # Create ECDF-niche based on personalized options:
#' n <- ecdf_nonnormal_niche(rho_vals = c(-0.7, -0.3, 0, 0.3, 0.7),
#'                           n_vals   = c(20L, 50L, 100L, 200L, 500L),
#'                           n_reps   = 10L,
#'                           N_ref    = 1e5,
#'                           seed     = 1991)
#'
#' @importFrom MASS mvrnorm
#' @importFrom ggplot2 ggplot aes geom_point scale_color_manual geom_vline geom_histogram
#'             theme_void theme element_rect labs facet_wrap theme_bw label_both xlim unit
#' @importFrom ggpp geom_plot_npc
#' @importFrom dplyr mutate group_by ungroup
#' @importFrom tidyr nest
#' @importFrom purrr map
#' @importFrom stats pnorm qnorm qweibull cov cov2cor mahalanobis pchisq ecdf cor
#' @import checkCLI
#'
#' @global data .data
#'
#' @export
ecdf_nonnormal_niche <- function(
    rho_vals = c(-0.7, -0.3, 0, 0.3, 0.7),
    n_vals   = c(20L, 50L, 100L, 200L, 500L),
    n_reps   = 10L,
    N_ref    = 1e5,
    temp_function = "qnorm",
    temp_parameters = list(mean = 20, sd = 5),
    prec_function = "qweibull",
    prec_parameters = list(shape = 2, scale = 10),
    seed     = NULL) {

  # Assertions
  assert_vector_cli(rho_vals, min.len = 1, unique = TRUE)
  assert_numeric_cli(rho_vals, lower = -1, upper = 1)
  assert_numeric_cli(n_vals, lower = 1)
  assert_vector_cli(n_vals, min.len = 1, unique = TRUE)
  assert_numeric_cli(n_reps, lower = 1)
  assert_numeric_cli(N_ref, lower = 1L, null.ok = TRUE)
  assert_numeric_cli(seed, lower = 0, null.ok = TRUE)

  assert_subset_cli(temp_function, c("qnorm", "qweibull"), empty.ok = FALSE)
  if(temp_function == "qnorm")    {tpars <- c("mean", "sd")}
  if(temp_function == "qweibull") {tpars <- c("shape", "scale")}
  assert_list_cli(temp_parameters, names = "strict",
                  unique = TRUE, len = 2, null.ok = FALSE,
                  any.missing = FALSE, all.missing = FALSE)
  assert_subset_cli(names(temp_parameters), tpars)
  if(temp_function == "qnorm") {
    assert_numeric_cli(temp_parameters$mean, len = 1)
    assert_numeric_cli(temp_parameters$sd, len = 1)
  }
  if(temp_function == "qweibull") {
    assert_numeric_cli(temp_parameters$shape, len = 1)
    assert_numeric_cli(temp_parameters$scale, len = 1)
  }

  assert_subset_cli(prec_function, c("qnorm", "qweibull"),  empty.ok = FALSE)
  if(prec_function == "qnorm")    {ppars <- c("mean", "sd")}
  if(prec_function == "qweibull") {ppars <- c("shape", "scale")}
  assert_list_cli(prec_parameters, names = "strict",
                  unique = TRUE, len = 2, null.ok = FALSE,
                  any.missing = FALSE, all.missing = FALSE)
  assert_subset_cli(names(prec_parameters), ppars)
  if(prec_function == "qnorm") {
    assert_numeric_cli(prec_parameters$mean, len = 1)
    assert_numeric_cli(prec_parameters$sd, len = 1)
  }
  if(prec_function == "qweibull") {
    assert_numeric_cli(prec_parameters$shape, len = 1)
    assert_numeric_cli(prec_parameters$scale, len = 1)
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # ------------------------------------------------------------------------------
  # 1. Set up the true distribution parameters (fixed across simulations)
  # ------------------------------------------------------------------------------
  ## Marginals
  # Temperature ~ Normal(mean = 20, sd = 5)
#  mu_temp <- ifelse(is.null(mu_temp), 20, mu_temp)
#  sd_temp <- ifelse(is.null(sd_temp), 5,  sd_temp)
#
#  # Precipitation ~ Weibull(shape = 2, scale = 10)
#  shape_precip <- ifelse(is.null(shape_precip), 2,  shape_precip)
#  scale_precip <- ifelse(is.null(scale_precip), 10, scale_precip)

  # ------------------------------------------------------------------------------
  # 2. Generate a huge reference sample to obtain the 'true' population parameters
  #    (mean vector and covariance matrix) for each value of rho
  # ------------------------------------------------------------------------------

  # Store 'true' parameters in a list indexed by rho values
  true_params <- list()


  for (rho in rho_vals) {
    ## Generate reference data
    # Specify a variance-covariance matrix for the Gaussian copula
    Sigma_cop <- matrix(c(1, rho, rho, 1), 2)

    # Generate bivariate normal data for each variable based on specified correlation
    z_ref <- MASS::mvrnorm(N_ref, mu = c(0, 0), Sigma = Sigma_cop)

    # Transform data into Cumulative Distribution: Uniform(0, 1)
    u_ref <- stats::pnorm(z_ref)

    # Transform the copula into marginal distributions
    if (temp_function == "qnorm") {
      temp_ref <- stats::qnorm(u_ref[,1], mean = temp_parameters$mean, sd = temp_parameters$sd) # generate marginal temperature
    } else if (temp_function == "qweibull") {
      temp_ref <- stats::qweibull(u_ref[,1], shape = temp_parameters$shape, scale = temp_parameters$scale) # generate marginal precipitation
    }

    if (prec_function == "qnorm") {
      precip_ref <- stats::qnorm(u_ref[,2], mean = prec_parameters$mean, sd = prec_parameters$sd) # generate marginal temperature
    } else if (prec_function == "qweibull") {
      precip_ref <- stats::qweibull(u_ref[,2], shape = prec_parameters$shape, scale = prec_parameters$scale) # generate marginal precipitation
    }

    X_ref <- cbind(temp_ref, precip_ref)

    # Calculate 'true' correlation matrix
    cov_true <- stats::cov(X_ref)
    cor_true <- stats::cov2cor(cov_true)

    true_params[[as.character(rho)]] <- list(
      mu    = colMeans(X_ref), # Calculate 'true' mean vector using marginal means
      Sigma = stats::cov(X_ref),  # Calculate 'true' variance-covariance matrix
      Cor   = cor_true           # Store 'true' correlation matrix
    )
  }


  # ------------------------------------------------------------------------------
  # 3. Simulation loop over rho, n, and replicates
  # ------------------------------------------------------------------------------

  # List to store correlation results
  cor_list <- list()
  obs_list <- list()
  rep_data <- list()

  idx_cor <- 1
  idx_obs <- 1

  for (rho in rho_vals) {
    # Generate sample from the copula model with the same rho
    Sigma_cop <- matrix(c(1, rho, rho, 1), 2)

    # Retrieve true parameters for this rho
    mu_true  <- true_params[[as.character(rho)]]$mu
    cov_true <- true_params[[as.character(rho)]]$Sigma

    for (n in n_vals) {
      for (rep in seq_len(n_reps)) {

        z <- MASS::mvrnorm(n, mu = c(0, 0), Sigma = Sigma_cop) # generate bivariate normal data for copula
        u <- stats::pnorm(z) # convert bivariate normal into cumulative distirbution

        if (temp_function == "qnorm") {
          temp <- stats::qnorm(u[,1], mean = temp_parameters$mean, sd = temp_parameters$sd) # generate marginal temperature
        } else if (temp_function == "qweibull") {
          temp <- stats::qweibull(u[,1], shape = temp_parameters$shape, scale = temp_parameters$scale) # generate marginal precipitation
        }

        if (prec_function == "qnorm") {
          precip <- stats::qnorm(u[,2], mean = prec_parameters$mean, sd = prec_parameters$sd) # generate marginal temperature
        } else if (prec_function == "qweibull") {
          precip <- stats::qweibull(u[,2], shape = prec_parameters$shape, scale = prec_parameters$scale) # generate marginal precipitation
        }

        X <- cbind(temp, precip) # combine into data frame

        # Calculate squared Mahalanobis distances using 'true' population parameters
        # Using 'true' values instead of sample values isolates the effect of non-normality
        # from estimation error
        D2 <- stats::mahalanobis(X, center = mu_true, cov = cov_true)

        # Calculate 'theoretical' suitability (based on chi-square)
        # This is 'wrong' here because distribution is non-normal, but to keep as a comparison
        S_chisq <- 1 - stats::pchisq(D2, df = 2)

        # Calculate 'empirical' suitability (based on ECDF)
        S_ecdf  <- 1 - stats::ecdf(D2)(D2)

        cor_list[[idx_cor]] <- data.frame(
          rho = rho,
          n   = n,
          rep = rep,
          cor = stats::cor(S_chisq, S_ecdf) # Calculate Pearson's correlation between the two suitabilities
        )

        obs_list[[idx_obs]] <- data.frame(
          rho = rho,
          n   = n,
          rep = rep,
          D2  = D2,
          S_chisq = S_chisq,
          S_ecdf  = S_ecdf,
          diff = S_ecdf - S_chisq
        )

        idx_cor <- idx_cor + 1
        idx_obs <- idx_obs + 1

        # Save the replicate for plotting
        rep_data[[paste0("rho", rho, "_n", n, "_rep", rep)]] <- data.frame(
          rho = rho,
          n = n,
          rep = rep,
          D2 = D2,
          S_chisq = S_chisq,
          S_ecdf = S_ecdf
          )
      }
    }
  }

  # ------------------------------------------------------------------------------
  # 4. Build graph
  # ------------------------------------------------------------------------------

  # Combine all results in a single data frame
  cor_df <- do.call(rbind, cor_list)
  obs_df <- do.call(rbind, obs_list)
  complete_obs <- do.call(rbind, rep_data)

  # Calculate difference between ECDF and Chi-square
  complete_obs <- complete_obs |>
    dplyr::mutate(diff = .data$S_ecdf - .data$S_chisq)

  # Create a data frame for inset histograms
  inset_plots <- complete_obs |>
    dplyr::group_by(n) |>
    tidyr::nest() |>
    dplyr::mutate(plot = purrr::map(data, ~ ggplot2::ggplot(.x, ggplot2::aes(x = .data$diff)) +
                                      ggplot2::xlim(c(-0.3, 0.3)) +
                                      ggplot2::geom_histogram(bins = 20, fill = "gray80", color = NA) +
                                      ggplot2::geom_vline(xintercept = 0, color = "red", linetype = "dashed") +
                                      ggplot2::theme_void() + # removes axes/background for a clean inset
                                      ggplot2::theme(plot.background = ggplot2::element_rect(fill = "white",
                                                                                             color = "black",
                                                                                             linewidth = 0.3))))

  # Create final plot
  suit_plot <- ggplot2::ggplot(complete_obs) +
    # Print suitability vs. Mahalanobis distance
    ggplot2::geom_point(ggplot2::aes(x = D2, y = S_ecdf, color = factor(rho)), size = 1, shape = 21) +
    ggplot2::scale_color_viridis_d(direction = -1) +
    ggplot2::geom_point(ggplot2::aes(x = .data$D2, y = .data$S_chisq), color = "red", size = 1, shape = 21) +

    # Print inset histograms
    ggpp::geom_plot_npc(data = inset_plots,
                        ggplot2::aes(npcx = 0.95, npcy = 0.65, label = plot),
                        vp.width = 0.3, vp.height = 0.25,
                        hjust = 1, vjust = 0) +

    ggplot2::facet_wrap(. ~ n, nrow = 1, labeller = ggplot2::label_both) +
    ggplot2::labs(color = expression(rho),
         x = expression("Squared Mahalanobis distance" ~ (D^2)),
         y = "Habitat Suitability") +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.key.size = ggplot2::unit(3, "point"))

  l <- list(
    suit_plot = suit_plot,
    cor_df = cor_df,
    obs_df = obs_df
  )
  return(l)
}

#' Create distance–suitability plot
#'
#' @param analysis_results List returned by \code{ecdf_theoretical_niche()}.
#' @return A ggplot object.
#'
#' @examples
#' # Create ECDF-niche based on personalized options:
#' res <- ecdf_theoretical_niche(n = 3,
#'                               n_population = 20000,
#'                               sample_sizes = seq(50, 1000, 50),
#'                               seed = 123)
#'
#' # Plot analysis results
#' create_distance_suitability_plot(res)
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_line scale_colour_manual theme_classic labs theme element_text
#' @importFrom stats pchisq ecdf
#' @import checkCLI
#'
#' @export
create_distance_suitability_plot <- function(analysis_results) {
  # Assertions
  assert_list_cli(analysis_results, len = 6)
  assert_names_cli(names(analysis_results), must.include = c("corplot", "sample_data",
                                                      "sample_niche", "chisq_suits",
                                                      "ecdf_suits", "mahal_dists"))
  assert_class_cli(analysis_results$corplot, classes = "ggplot")
  assert_matrix_cli(analysis_results$sample_data)
  assert_numeric_cli(analysis_results$sample_niche, lower = 0, upper = 1)
  assert_numeric_cli(analysis_results$chisq_suits, lower = 0, upper = 1)
  assert_numeric_cli(analysis_results$ecdf_suits, lower = 0, upper = 1)
  assert_numeric_cli(analysis_results$mahal_dists, lower = 0)

  plot_data <- data.frame(
    Mahalanobis_Distance   = analysis_results$mahal_dists,
    Niche_Suitability      = analysis_results$sample_niche,
    ChiSquared_suitability = analysis_results$chisq_suits,
    ECDF_suitability       = analysis_results$ecdf_suits
  )

  ggplot2::ggplot(plot_data, ggplot2::aes(x = .data$Mahalanobis_Distance)) +
    ggplot2::geom_point(
      ggplot2::aes(y = .data$Niche_Suitability, color = "Niche Records"),
      shape = 21, size = 3
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = .data$ChiSquared_suitability, color = "1 - Chi-squared"),
      linewidth = 1
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = .data$ECDF_suitability, color = "1 - ECDF"),
      linewidth = 1
    ) +
    ggplot2::scale_colour_manual(
      values = c("Niche Records" = "black",
                 "1 - Chi-squared" = "cyan4",
                 "1 - ECDF" = "orange")
    ) +
    ggplot2::theme_classic() +
    ggplot2::labs(
      x = "Mahalanobis Distance",
      y = "Environmental Suitability",
      title = paste0(ncol(analysis_results$sample_data), " Predictor Variables")
    ) +
    ggplot2::ylim(0, 1) +
    ggplot2::theme(
      axis.title  = ggplot2::element_text(size = 12),
      axis.text   = ggplot2::element_text(size = 10),
      legend.title = ggplot2::element_blank(),
      legend.position = c(0.8, 0.8)
    )
}

#' Run full ECDF–Mahalanobis analysis
#'
#' Convenience function that reproduces the three figures from the original
#' manuscript for 1–5 dimensions.
#'
#' @param dims Integer vector of dimensions (default 1:5).
#' @param seed Optional seed for reproducibility.
#'
#' @return A list containing:
#' \itemize{
#'   \item analyses: list of ecdf_theoretical_niche() outputs.
#'   \item figure1, figure2, figure3: grobs with arranged plots.
#' }
#'
#' @examples
#' # Recreate original manuscript output:
#' set.seed(3)
#' full_res <- run_ecdf_mahal_analysis(dims = 1:5)
#'
#' @importFrom ggplot2 ggplot aes geom_point labs scale_colour_viridis_c theme_classic theme element_blank
#' @importFrom lemon grid_arrange_shared_legend
#' @import checkCLI
#'
#' @global .data
#'
#' @export
run_ecdf_mahal_analysis <- function(dims = 1:5, seed = 3L) {
  # Assertions
  assert_vector_cli(dims, min.len = 1, unique = TRUE)
  assert_numeric_cli(dims, lower = 1)
  assert_numeric_cli(seed, lower = 0, null.ok = TRUE)

  if (!is.null(seed)) set.seed(seed)

  analyses <- lapply(dims, function(d) ecdf_theoretical_niche(d, seed = seed))

  vars1 <- analyses[[1L]]
  vars2 <- analyses[[2L]]
  vars3 <- analyses[[3L]]
  vars4 <- analyses[[4L]]
  vars5 <- analyses[[5L]]

  # Figure 1 (2D spatial)
  spatial_data <- data.frame(
    Variable_1             = vars2$sample_data[, 1],
    Variable_2             = vars2$sample_data[, 2],
    Niche_suitability      = vars2$sample_niche,
    ChiSquared_suitability = vars2$chisq_suits,
    ECDF_suitability       = vars2$ecdf_suits
  )

  common_theme <- ggplot2::theme_classic() +
    ggplot2::theme(legend.position = "none")

  plot_niche <- ggplot2::ggplot(spatial_data,
                                ggplot2::aes(x = .data$Variable_1, y = .data$Variable_2)) +
    ggplot2::geom_point(ggplot2::aes(color = .data$Niche_suitability), size = 3) +
    ggplot2::labs(
      x = "",
      y = "Variable 2",
      title = "Simulated Niche",
      color = "Environmental Suitability"
    ) +
    ggplot2::scale_colour_viridis_c() +
    common_theme

  plot_chisq <- ggplot2::ggplot(spatial_data,
                                ggplot2::aes(x = .data$Variable_1, y = .data$Variable_2)) +
    ggplot2::geom_point(
      ggplot2::aes(color = .data$ChiSquared_suitability), size = 3
    ) +
    ggplot2::labs(
      x = "Variable 1",
      y = "",
      title = "1 - Chi-squared Suitability"
    ) +
    ggplot2::scale_colour_viridis_c() +
    common_theme

  plot_ecdf <- ggplot2::ggplot(spatial_data,
                               ggplot2::aes(x = .data$Variable_1, y = .data$Variable_2)) +
    ggplot2::geom_point(
      ggplot2::aes(color = .data$ECDF_suitability), size = 3
    ) +
    ggplot2::labs(
      x = "",
      y = "",
      title = "1 - ECDF Suitability"
    ) +
    ggplot2::scale_colour_viridis_c() +
    common_theme

  figure1 <- lemon::grid_arrange_shared_legend(
    plot_niche, plot_chisq, plot_ecdf,
    ncol = 3, nrow = 1, position = "bottom"
  )

  # Figure 2: correlation plots
  figure2 <- lemon::grid_arrange_shared_legend(
    vars1$corplot + ggplot2::labs(x = "", y = "Correlation") +
      ggplot2::coord_cartesian(ylim = c(0.95, 1)),
    vars2$corplot + ggplot2::labs(x = "", y = "") +
      ggplot2::coord_cartesian(ylim = c(0.95, 1)),
    vars3$corplot + ggplot2::labs(x = "Number of Records", y = "") +
      ggplot2::coord_cartesian(ylim = c(0.95, 1)),
    vars4$corplot + ggplot2::labs(x = "", y = "") +
      ggplot2::coord_cartesian(ylim = c(0.95, 1)),
    vars5$corplot + ggplot2::labs(x = "", y = "") +
      ggplot2::coord_cartesian(ylim = c(0.95, 1)),
    ncol = 5, nrow = 1, position = "bottom"
  )

  # Figure 3: distance–suitability relationships
  distance_plot_1 <- create_distance_suitability_plot(vars1)
  distance_plot_2 <- create_distance_suitability_plot(vars2)
  distance_plot_3 <- create_distance_suitability_plot(vars3)
  distance_plot_4 <- create_distance_suitability_plot(vars4)
  distance_plot_5 <- create_distance_suitability_plot(vars5)

  figure3 <- lemon::grid_arrange_shared_legend(
    distance_plot_1 + ggplot2::labs(x = "", y = "Environmental Suitability"),
    distance_plot_2 + ggplot2::labs(x = "", y = ""),
    distance_plot_3 + ggplot2::labs(x = "Mahalanobis Distance", y = ""),
    distance_plot_4 + ggplot2::labs(x = "", y = ""),
    distance_plot_5 + ggplot2::labs(x = "", y = ""),
    ncol = 5, nrow = 1, position = "bottom"
  )

  list(
    analyses = analyses,
    figure1  = figure1,
    figure2  = figure2,
    figure3  = figure3
  )
}

# Generated by roxyglobals: do not edit by hand

utils::globalVariables(c(
  # <ecdf_theoretical_niche>
  "",
  # <ecdf_nonnormal_niche>
  # <run_ecdf_mahal_analysis>
  ".data",
  # <ecdf_compare_niche>
  "cor",
  # <ecdf_nonnormal_niche>
  "data",
  NULL
))

---
title: "ECDF and Mahalanobis Distance for Empirical Niche Modeling"
author: "Luíz Fernando Esser"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ecdfniche_comparison}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4
)
```

## Introduction

This vignette shows how to use the **ECDFniche** package to reproduce the simulations from the original manuscript, comparing Mahalanobis distance–based suitability transformations using the chi-squared distribution and the empirical cumulative distribution function (ECDF).

We follow a *virtual ecologist* approach (Zurell et al. 2010) to evaluate how different transformations of the Mahalanobis distance recover a simulated niche across varying dimensionalities (number of predictor variables) and sample sizes (number of records), and then extend the analysis to a bivariate non-normal environmental space.

```{r}
library(ECDFniche)
```

## Multivariate normal niche: `ecdf_compare_niche()`

In the first set of simulations, we assume that the environmental predictors describing a species’ niche follow a multivariate normal distribution.

Dimensions \(p\) range from 1 to 5, and sample sizes \(n\) range from 20 to 500 in steps of 20, mimicking increasing numbers of occurrence records.

For each combination of \(p\) and \(n\), `ecdf_compare_niche()`:

1. Draws a fixed covariance matrix \(\Sigma\) by sampling from a Wishart distribution with \(p + 2\) degrees of freedom and scale matrix set to zero with a diagonal of 1, ensuring a positive-definite covariance for that \((p, n)\) combination.
2. Generates 30 independent replicates of \(n\) records from a \(p\)-variate normal distribution with mean vector \(\mu_p = \mathbf{0}\) (interpreted as the environmental optimum) and covariance matrix \(\Sigma\).
3. For each replicate, estimates the sample mean \(\mu_r\) and sample covariance \(\Sigma_r\) and computes the squared Mahalanobis distance
\[
D^2 = (x - \mu_r)^\top \Sigma_r^{-1} (x - \mu_r).
\]
4. Computes two suitability metrics for every record:
   - Theoretical chi-squared suitability:
\[
S_{\chi^2} = 1 - F_{\chi^2_p}(D^2),
\]
   where \(F_{\chi^2_p}\) is the cumulative distribution function of the chi-squared distribution with \(p\) degrees of freedom.
   
   - Empirical ECDF-based suitability:
\[
S_{\text{ECDF}} = 1 - F_{\text{ECDF}}(D^2),
\]
   where \(F_{\text{ECDF}}\) is the empirical cumulative distribution function of the distances.
   
5. Calculates Pearson’s correlation coefficient between \(S_{\chi^2}\) and \(S_{\text{ECDF}}\) for each replicate.

This setup mimics a realistic ENM scenario where multivariate means and true covariances are unknown and must be estimated from occurrence records drawn from a correlated environmental space representing the species’ theoretical niche (sensu Hutchinson 1978).

```{r}
set.seed(1991)
normal_res <- ecdf_compare_niche(
  p_vals = 1:5,
  n_vals = seq(20L, 500L, 20L),
  n_reps = 30L
)
```

### Figure 1: Correlation vs sample size

`cor_plot` summarizes, for each \(p\) and \(n\), the mean and standard deviation of the correlation between chi-squared and ECDF suitabilities across the 30 replicates, with individual replicate values shown as grey points. The plot shows the correlation between suitability metrics estimated using the chi-squared and the Empirical Cumulative Distribution Function (ECDF) across sample sizes and numbers of environmental predictors. Each panel shows Pearson correlation coefficients as a function of the number of occurrence records for a given dimensionality p (1 to 5 variables). Correlations are generally very high (rarely < 0.95), increasing with sample size and slightly increasing with dimensionality.

```{r, fig.width=11, fig.height=3}
normal_res$cor_plot
```

### Figure 2: Suitability vs Mahalanobis distance

`suit_plot` pools observations across sample sizes for each \(p\), recomputes an ECDF-based suitability on the combined distances, and compares these to chi-squared curves. The plot shows the relationship between squared Mahalanobis distance (x-axis) and environmental suitability metrics (y-axis) estimated using the chi-squared and the Empirical Cumulative Distribution Function (ECDF) across different numbers of environmental predictors. Grey curves represent the habitat suitability based on the chi-squared distribution while red circles represent habitat suitability via ECDF, highlighting that ECDF closely tracks the theoretical chi‑squared mapping over the distance range.

```{r, fig.width=11, fig.height=3}
normal_res$suit_plot
```

## Non-normal bivariate niche: `ecdf_nonnormal_niche()`

In many real ENM applications, environmental covariates are not jointly normal and species can show complex responses to environmental gradients (Anderson et al. 2022).

To explore this, `ecdf_nonnormal_niche()` simulates a bivariate environmental space for temperature and precipitation using a Gaussian copula:

- Temperature follows a normal distribution with mean 20 °C and standard deviation 5 °C.
- Precipitation follows a Weibull distribution with shape 2 and scale 10 mm, representing skewed rainfall data.
- The dependence between the two variables is controlled by a correlation parameter \(\rho \in \{-0.7, -0.3, 0, 0.3, 0.7\}\), reflecting negative to positive associations observed in climatological studies (e.g. Anderson et al. 2019).

For each \(\rho\), the function:

1. Uses a Gaussian copula with correlation \(\rho\) to generate a large reference sample of size \(N_{\text{ref}}\) (default \(10^5\)) and derive “true” population mean vector and covariance matrix for the bivariate distribution.
2. For each combination of \(\rho\) and sample size \(n \in \{20, 50, 100, 200, 500\}\), draws \(n_{\text{reps}}\) replicate samples directly from the copula-based bivariate non-normal distribution.
3. Computes squared Mahalanobis distances \(D^2\) using the *true* mean and covariance from the reference sample, isolating the effect of non-normality from parameter estimation error.
4. Calculates:
   - Chi-squared suitability \(S_{\chi^2} = 1 - F_{\chi^2_2}(D^2)\), which is theoretically “wrong” under non-normality but serves as the standard parametric benchmark.
   - ECDF-based suitability \(S_{\text{ECDF}} = 1 - F_{\text{ECDF}}(D^2)\).
5. Returns per-replicate correlations and observation-level data, plus a summary plot comparing the two suitabilities.

```{r}
set.seed(1991)
nonnormal_res <- ecdf_nonnormal_niche(
  rho_vals = c(-0.7, -0.3, 0, 0.3, 0.7),
  n_vals   = c(20L, 50L, 100L, 200L, 500L),
  n_reps   = 10L,
  N_ref    = 1e5,
  temp_function = "qnorm",
  temp_parameters = list(mean = 20, sd = 5),
  prec_function = "qweibull",
  prec_parameters = list(shape = 2, scale = 10)
)
```

### Figure 3: Suitability vs distance under non-normality

`suit_plot` shows ECDF-based suitability (colored by \(\rho\)) and chi-squared suitability (red points) as functions of \(D^2\), faceted by sample size. Plots highlight the sensitivity of chi-squared and ECDF suitability metrics to sample size and variable correlation in non-normal bivariate data. Internal histograms represent the distribution of ECDF-based values around the chi-squared-based suitability. The ECDF estimator shows higher stochasticity in small samples but converges to the chi-squared expectation in larger samples.


```{r, fig.width=11, fig.height=3}
nonnormal_res$suit_plot
```

## Customizing simulations

You can customize key aspects of both simulations to reproduce or extend the analyses:

- **Multivariate normal niche** (`ecdf_compare_niche()`):
  - `p_vals`: set of dimensions (e.g. `1:10`) to evaluate high-dimensional behavior.
  - `n_vals`: sample size grid, e.g. smaller `n` to explore the effect of limited records.
  - `n_reps`: number of replicates per combination for more stable summaries.

```{r, fig.width=11, fig.height=3}
res_custom_normal <- ecdf_compare_niche(
  p_vals = 2:4,
  n_vals = seq(20L, 200L, 20L),
  n_reps = 50L,
  seed   = 42
)
res_custom_normal$cor_plot
```

- **Non-normal bivariate niche** (`ecdf_nonnormal_niche()`):
  - `rho_vals`: alternative dependence structures.
  - `n_vals`, `n_reps`: sample size and replication design.
  - `N_ref`: reference population size; larger values approximate the true parameters more closely.

```{r, fig.width=11, fig.height=3}
res_custom_nonnorm <- ecdf_nonnormal_niche(
  rho_vals = c(-0.5, 0, 0.5),
  n_vals   = c(30L, 100L, 300L),
  n_reps   = 20L,
  N_ref    = 5e4,
  temp_function = "qnorm",
  temp_parameters = list(mean = 20, sd = 5),
  prec_function = "qweibull",
  prec_parameters = list(shape = 2, scale = 10),
  seed     = 123
)
res_custom_nonnorm$suit_plot
```

Together, these simulations illustrate when the traditional chi-squared transformation is reliable and when the ECDF-based approach provides a more robust mapping from niche-based distances to habitat suitability, particularly when environmental covariates deviate from multivariate normality.

---
title: "ECDF and Mahalanobis Distance for Theoretical Niche Modeling"
author: "Luíz Fernando Esser"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ecdfniche}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>",
fig.width = 7,
fig.height = 4
)
```

## Introduction

This vignette shows how to use the **ECDFniche** package to reproduce the simulations from the original `ECDF_MahalDist.R` script, comparing Mahalanobis distance-based suitability transformations using the chi-squared distribution and the empirical cumulative distribution function (ECDF).

```{r}
library(ECDFniche)
```

## Core simulation: `ecdf_theoretical_niche()`

The function `ecdf_theoretical_niche()` simulates a multivariate normal "environmental space", computes Mahalanobis distances for a sample of points, and then maps those distances to suitability using:

- \(1 - F_{\chi^2}(D^2)\) (theoretical chi-squared transformation)
- \(1 - \text{ECDF}(D^2)\) (empirical transformation)

```{r}
set.seed(3)
res1 <- ecdf_theoretical_niche(n = 2)
res1$corplot
```

The returned list contains:

- `corplot`: correlation vs sample size between the “true” niche and both suitability transformations
- `sample_data`: matrix with the last sample of environmental predictors
- `sample_niche`, `chisq_suits`, `ecdf_suits`: suitability values
- `mahal_dists`: Mahalanobis distances for the last sample 

You can directly plot the correlation object:

```{r}
res1$corplot
```


## Reproducing the full analysis

The convenience function `run_ecdf_mahal_analysis()` wraps the original workflow: it runs `ecdf_theoretical_niche()` for several dimensions (by default 1 to 5) and produces three figures analogous to those in the script.

```{r}
set.seed(3)
full_res <- run_ecdf_mahal_analysis(dims = 1:5)
```


### Figure 1: Spatial visualization (2D)

Figure 1 shows the 2D environmental space (two predictor variables) with color representing different suitability definitions: the simulated "true" niche, the chi-squared-based suitability, and the ECDF-based suitability.

```{r}
full_res$figure1 |> plot()
```


### Figure 2: Correlation vs sample size

Figure 2 presents, for each dimensionality, how the correlation between the true niche and each distance-to-suitability transformation changes with sample size.

```{r}
full_res$figure2 |> plot()
```


### Figure 3: Distance–suitability relationships

Figure 3 plots Mahalanobis distance on the x-axis and suitability on the y-axis, showing how niche records, chi-squared suitability, and ECDF suitability relate across different numbers of predictor variables.

```{r}
full_res$figure3 |> plot()
```


## Customizing simulations

You can customize key aspects of the simulation by passing arguments to
`ecdf_theoretical_niche()`:

```{r}
res_custom <- ecdf_theoretical_niche(
n = 3,
n_population = 20000,
sample_sizes = seq(50, 1000, 50),
seed = 123
)

res_custom$corplot
```

These arguments control the dimensionality, the size of the
environmental "background", and the grid of sample sizes used to compute
correlations.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ecdf_compare_niche.R
\name{ecdf_compare_niche}
\alias{ecdf_compare_niche}
\title{Simulations and analyses of Mahalanobis distance-based habitat suitability}
\usage{
ecdf_compare_niche(
  p_vals = 1:5,
  n_vals = seq(20L, 500L, 20L),
  n_reps = 30L,
  seed = NULL
)
}
\arguments{
\item{p_vals}{Integer vector; number of predictor variables (dimensions).}

\item{n_vals}{Integer vector; number of records (sample sizes).}

\item{n_reps}{Integer; number of replicates per combination.}

\item{seed}{Optional integer for reproducibility.}
}
\value{
A list with:
\itemize{
\item cor_plot: ggplot of correlation vs sample size.
\item suit_plot: ggplot of suitability vs Mahalanobis distance.
\item cond_plot: ggplot of correlation vs condition number.
\item cor_df: raw correlation data.
\item obs_df: observation-level data.
\item cov_df: covariance diagnostics.
}
}
\description{
The objective is to compare the performance of habitat suitability calculated
based on chi-squared cumulative distribution function and Empirical
Cumulative Distribution Function (ECDF)
}
\details{
Performs replicated simulations of multivariate normal data to evaluate
the agreement between suitability derived from chi-squared distribution
and empirical cumulative distribution function (ECDF).
}
\examples{
# Create ECDF-niche based on personalized options:
n <- ecdf_compare_niche(p_vals = 1:3,
                        n_vals = seq(50L, 500L, 50L),
                        n_reps = 10L,
                        seed = 1991)

}
\author{
Matheus T. Baumgartner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ecdf_theoretical_niche.R
\name{ecdf_theoretical_niche}
\alias{ecdf_theoretical_niche}
\title{Niche analysis using ECDF and chi-squared}
\usage{
ecdf_theoretical_niche(
  n,
  n_population = 10000L,
  sample_sizes = seq(20L, 500L, 20L),
  seed = NULL
)
}
\arguments{
\item{n}{Integer; number of predictor variables (dimensions).}

\item{n_population}{Integer; size of simulated environmental population.}

\item{sample_sizes}{Integer vector of sample sizes to evaluate.}

\item{seed}{Optional integer seed for reproducibility.}
}
\value{
A list with:
\itemize{
\item corplot: ggplot object with correlation vs sample size.
\item sample_data: matrix of simulated sample points.
\item sample_niche: numeric vector of “true” niche suitability.
\item chisq_suits: numeric vector, 1 - pchisq(Mahalanobis).
\item ecdf_suits: numeric vector, 1 - ECDF(Mahalanobis).
\item mahal_dists: numeric vector of Mahalanobis distances.
}
}
\description{
Simulate niche suitability from Mahalanobis distance using both
chi-squared and empirical CDF transformations, for a given number
of predictor variables.
}
\examples{
# Create ECDF-niche based on personalized options:
n <- ecdf_theoretical_niche(n = 3,
                            n_population = 20000,
                            sample_sizes = seq(50, 1000, 50),
                            seed = 123)

}
\author{
Luíz Fernando Esser
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{run_ecdf_mahal_analysis}
\alias{run_ecdf_mahal_analysis}
\title{Run full ECDF–Mahalanobis analysis}
\usage{
run_ecdf_mahal_analysis(dims = 1:5, seed = 3L)
}
\arguments{
\item{dims}{Integer vector of dimensions (default 1:5).}

\item{seed}{Optional seed for reproducibility.}
}
\value{
A list containing:
\itemize{
\item analyses: list of ecdf_theoretical_niche() outputs.
\item figure1, figure2, figure3: grobs with arranged plots.
}
}
\description{
Convenience function that reproduces the three figures from the original
manuscript for 1–5 dimensions.
}
\examples{
# Recreate original manuscript output:
set.seed(3)
full_res <- run_ecdf_mahal_analysis(dims = 1:5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{create_distance_suitability_plot}
\alias{create_distance_suitability_plot}
\title{Create distance–suitability plot}
\usage{
create_distance_suitability_plot(analysis_results)
}
\arguments{
\item{analysis_results}{List returned by \code{ecdf_theoretical_niche()}.}
}
\value{
A ggplot object.
}
\description{
Create distance–suitability plot
}
\examples{
# Create ECDF-niche based on personalized options:
res <- ecdf_theoretical_niche(n = 3,
                              n_population = 20000,
                              sample_sizes = seq(50, 1000, 50),
                              seed = 123)

# Plot analysis results
create_distance_suitability_plot(res)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mahal.dist.R
\docType{data}
\name{mahal.dist}
\alias{mahal.dist}
\title{Mahalanobis Distance Classifier for Ecological Niche Modeling}
\format{
An object of class \code{list} of length 12.
}
\usage{
mahal.dist
}
\description{
A custom \code{caret} model specification implementing a Mahalanobis
distance-based classifier for ecological niche modeling (ENM) and
species distribution modeling (SDM). This implementation supports both
parametric (chi-squared) and nonparametric (empirical cumulative
distribution function; ECDF) transformations of Mahalanobis distances
into suitability scores.
}
\details{
The model is trained using presence-only data to estimate the centroid
and covariance structure of environmental conditions associated with
species occurrences. Suitability is then derived as the inverse tail
probability of the Mahalanobis distance between new observations and
the estimated niche centroid.

Two approaches are available to transform Mahalanobis distances into
probabilities:
\itemize{
\item \code{"chisq"}: assumes distances follow a chi-squared
distribution with degrees of freedom equal to the number of predictors.
\item \code{"ecdf"}: uses the empirical cumulative distribution
function of training distances, providing a nonparametric estimate
of suitability.
}

The ECDF-based approach is particularly useful when the assumption of
multivariate normality is violated, which is common in ecological data.

This model can be used within the \code{caret::train()} framework,
enabling resampling, tuning, and ensemble modeling workflows for
ecological niche modeling.
}
\section{Model Parameters}{

\describe{
\item{abs}{Logical. If \code{TRUE}, predictions are binarized using a
fixed threshold (default: 0.05). If \code{FALSE}, the class with the
highest predicted probability is returned.}

\item{method}{Character. Method used to convert Mahalanobis distances
into suitability values. Options are \code{"chisq"} or \code{"ecdf"}.}
}
}

\section{Details}{

The Mahalanobis distance defines an ellipsoidal niche in environmental
space. Under the chi-squared formulation, suitability decreases as the
distance from the niche centroid increases. The ECDF formulation
relaxes distributional assumptions by estimating suitability directly
from the empirical distribution of distances observed in presence data.

Predictions return class probabilities for \code{"presence"} and
\code{"pseudoabsence"}, allowing flexible thresholding and ensemble
integration.
}

\section{Usage in caret}{

This object can be supplied to \code{caret::train()} as a custom model:

\preformatted{
library(caret)

model <- train(
  x = predictors,
  y = response,
  method = mahal.dist,
  trControl = trainControl(classProbs = TRUE)
)
}

You can also run only ECDF by adjusting the tuning grid:
\preformatted{
library(caret)

grid <- expand.grid(
  abs = c(TRUE, FALSE),
  method = "ecdf"
)

model <- train(
  x = predictors,
  y = response,
  method = mahal.dist,
  tuneGrid = grid,
  trControl = trainControl(classProbs = TRUE)
)
}
}

\seealso{
\code{\link[stats]{mahalanobis}}, \code{\link[stats]{ecdf}},
\code{\link[caret]{train}}
}
\keyword{mahalanobis}
\keyword{models}
\keyword{niche-modeling}
\keyword{species-distribution-modeling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ecdf_nonnormal_niche.R
\name{ecdf_nonnormal_niche}
\alias{ecdf_nonnormal_niche}
\title{Compare ECDF and Chi-square suitability under non-normal data}
\usage{
ecdf_nonnormal_niche(
  rho_vals = c(-0.7, -0.3, 0, 0.3, 0.7),
  n_vals = c(20L, 50L, 100L, 200L, 500L),
  n_reps = 10L,
  N_ref = 1e+05,
  temp_function = "qnorm",
  temp_parameters = list(mean = 20, sd = 5),
  prec_function = "qweibull",
  prec_parameters = list(shape = 2, scale = 10),
  seed = NULL
)
}
\arguments{
\item{rho_vals}{Numeric vector; correlations between variables.}

\item{n_vals}{Integer vector; sample sizes.}

\item{n_reps}{Integer; number of replicates.}

\item{N_ref}{Integer; size of reference population for "true" parameters.}

\item{temp_function}{Character; function used to model temperature values. One of: "qnorm" or
"qweibull".}

\item{temp_parameters}{List; list organizing parameters to pass to \code{temp_function}.}

\item{prec_function}{Character; function used to model precipitation values. One of: "qnorm" or
"qweibull".}

\item{prec_parameters}{List; list organizing parameters to pass to \code{temp_function}.}

\item{seed}{Optional integer for reproducibility.}
}
\value{
A list with:
\itemize{
\item suit_plot: ggplot of suitability vs Mahalanobis distance
\item cor_df: correlation results
\item obs_df: observation-level data
}
}
\description{
Script to run a simulation study to compare Chi-square vs. ECDF approaches
to quantify habitat suitability based on bivariate non-normal data.
Bivariate data was simulated based on environmental variables (temperature
and precipitation) using Gaussian copulas. Temperature followed a normal
distribution while precipitation followed a Weibull distribution.
The choices of the distributions were based on Haddad (2021) - Theoretical
and Applied Climatology (for temperature) and on the estimation of rainfall
in milimeters by Wilks (1989) - Journal of Applied Meteorology.
Because the relationship between temperature and precipitation is complex
across space (Rodrigo, 2022 - Theoretical and Applied Climatology), we
defined five correlation values between the two variables.

\code{temp_parameters} and \code{prec_parameters} must comply to \code{stats::qnorm} or
\code{stats::qweibull}, depending on the function chosen on \code{temp_function} and
\code{prec_function}. For \code{"qnorm"}, user can specify \code{mean} and \code{sd}, while
for \code{"qweibull"}
}
\details{
Simulates bivariate environmental data using Gaussian copulas with
non-normal marginals (Normal for temperature and Weibull for precipitation),
and evaluates agreement between chi-squared and ECDF suitability.
}
\examples{
# Create ECDF-niche based on personalized options:
n <- ecdf_nonnormal_niche(rho_vals = c(-0.7, -0.3, 0, 0.3, 0.7),
                          n_vals   = c(20L, 50L, 100L, 200L, 500L),
                          n_reps   = 10L,
                          N_ref    = 1e5,
                          seed     = 1991)

}
\author{
Matheus T. Baumgartner
}
